# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

import os
from _snap_ml.utils import _is_mpi_enabled
import warnings

def _can_allow_cuml():
    """
       Perform basic validations whether cuML can be enabled or not.
       More validations can be done by individual algorithm level.
    """

    # Check if cuml is installed
    try:
        import cuml
    except ImportError:
        return False

    import sys
    if sys.version_info.major != 3:
        warnings.warn("cuML in pai4sk is supported only with python 3.6 and python 3.7.", UserWarning)
        return False

    if _is_mpi_enabled():
        warnings.warn("MPI based distributed model training is currently not supported" \
                      " in pai4sk for cuML based algorithms.", UserWarning)
        return False

    return True

def _is_cuml_algorithm(algorithm, allowed_algos=[]):
    """
       Check if the given `algorithm` is supported by cuML for this model.
    """

    if algorithm != 'auto' and algorithm != 'cuml' and algorithm not in allowed_algos:
        extra_algos = ""
        if allowed_algos:
            extra_algos = "'{0}'".format("', '".join(extra_algos)) + ", "
            warnings.warn("This model of cuML can be used in pai4sk only when " \
                          "'algorithm' parameter is set to {}'cuml' or 'auto'." \
                          .format(extra_algos), UserWarning)
        return False

    return True

def _do_not_allow_cuml(X, algorithm="auto", cuml_algos=["cuml"]):
    """Raise error if X is a cudf dataframe or
       cuml-specific arguments are seen.
       algorithm: algorithm parameter set by user for this model.
       cuml_algos: list of algorithm values allowed only with cuml for a specific model.
    """
    if algorithm in cuml_algos:
        raise ValueError("Some prerequisite of cuML in pai4sk is not satisfied. " \
                         "As cuml-specific algorithm {} is passed as parameter, " \
                         "scikit-learn can't perform training. See the API " \
                         "documentation for more details.".format(algorithm))

    try:
        import cudf
    except ImportError:
        return

    if isinstance(X, cudf.DataFrame):
        raise ValueError("Some prerequisite of cuML in pai4sk is not satisfied. " \
                         "As cudf dataframe is given as input, scikit-learn " \
                         "can't perform training on it. See the API " \
                         "documentation for more details.")

