# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function

import time 
import argparse

# pai4sk explicit imports for snap_ml APIs
from pai4sk import MPI
from pai4sk import sml_io
from pai4sk import LinearRegression
from pai4sk.sml_metrics import mean_squared_error as snapMSE

# pai4sk implicit API imports which use snap_ml or sklearn
from pai4sk.linear_model import Ridge
from pai4sk.metrics import mean_squared_error

# pai4sk explicit imports for sklearn APIs
from pai4sk.linear_model import RidgeSklearn
from pai4sk.metrics import mean_squared_errorSklearn
from pai4sk.datasets import load_svmlight_fileSklearn

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
  default=defaultPath
)
CLI.add_argument(
   "--device_ids",
   nargs="*",
   type=int,
   default=[]
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
args = CLI.parse_args()

(comm_size, rank) = MPI.Comm_get_info()

data_x,data_y = sml_io.load_snap_file("data/small_sparse.snap")

clf_snap = LinearRegression(device_ids=args.device_ids, use_gpu=args.use_gpu, max_iter=15)

# Training
t0 = time.time()
clf_snap.fit(data_x,data_y)
if (rank == 0):
    print("[pai4sk explicit] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

# Inference
pred_vals = clf_snap.predict(data_x)

# Evaluate log-loss on test set
if (rank == 0):
    print("[pai4sk explicit] MPI process %d : Mean squared error:    %.4f" % (rank, snapMSE(data_y, pred_vals)))

clf = Ridge(device_ids=args.device_ids, use_gpu=args.use_gpu, max_iter=15,
            dual=True, fit_intercept=False)

# Training
t0 = time.time()
clf.fit(data_x,data_y)
if (rank == 0):
    print("[pai4sk implicit] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

# Inference
pred_vals = clf.predict(data_x)

# Evaluate log-loss on test set
if (rank == 0):
    print("[pai4sk implicit] MPI process %d : Mean squared error:    %.4f" % (rank, mean_squared_error(data_y, pred_vals)))

if args.skip_sklearn:
    if (rank == 0):
        print('Execution using scikit-learn is skipped.')
    exit()

X,y = load_svmlight_fileSklearn(args.data_path + "/data/small.libsvm")

# Create a LogisticRegression (dual is faster for this application)
clf = RidgeSklearn(fit_intercept=False)

# Training time
t0 = time.time()
clf.fit(X, y)

if (rank == 0):
    print("[sklearn] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

# Inference
pred_vals = clf.predict(X)

# Evaluate log-loss on test set
L2loss_sklearn = mean_squared_errorSklearn(y, pred_vals)
if (rank == 0):
    print("[sklearn] MPI process %d : Mean squared error:    %.4f" % (rank, L2loss_sklearn))
